// idrapp.cpp--class functions for Idrapp

#include <windows.h>
#include "idrapp.h"

// basic API function types

typedef short (CALLBACK* IsIdrisiPresentFuncType)();
typedef short (CALLBACK* LaunchIdrisiFuncType)(short);
typedef short (CALLBACK* SetDataDirectoryFuncType)(char *);
typedef short (CALLBACK* RegisterClientFuncType)();
typedef short (CALLBACK* LaunchModuleFuncType)(short, short, char *, char *, char *, char *, short *);
typedef short (CALLBACK* GetProgressFuncType)(short, short, ProgStruct *);
typedef short (CALLBACK* UnRegisterClientFuncType)(short);
typedef short (CALLBACK* CloseIdrisiFuncType)();

// extended API function types

typedef short (CALLBACK* SetDebugModeFuncType)(short);
typedef short (CALLBACK* GetIdrisiHwndFuncType)();
typedef short (CALLBACK* GetIdrisiDirectoryFuncType)(char *);
typedef short (CALLBACK* GetIdrisiLanguageFuncType)(char *);
typedef short (CALLBACK* GetDataDirectoryFuncType)(char *);
typedef short (CALLBACK* InitProgressTrackingFuncType)(short, char *);
typedef short (CALLBACK* SetProgressFuncType)(short, short, ProgStruct *);
typedef short (CALLBACK* DisplayCompositionFuncType)(short, short, char *, char *);
typedef short (CALLBACK* Read_DocFileFuncType)(char *, ImageDoc *);
typedef short (CALLBACK* Create_DocFileFuncType)(char *, ImageDoc *);
typedef short (CALLBACK* Read_Vec_DocFileFuncType)(char *, VectorDoc *);
typedef short (CALLBACK* Create_Vec_DocFileFuncType)(char *, VectorDoc *);
typedef short (CALLBACK* Read_Val_DocFileFuncType)(char *, ValuesDoc *);
typedef short (CALLBACK* Create_Val_DocFileFuncType)(char *, ValuesDoc *);
typedef short (CALLBACK* Read_LegendFuncType)(char *, short, short, char *);
typedef short (CALLBACK* Add_LegendFuncType)(char *, short, short, char *);
typedef short (CALLBACK* GetIdrisiExtensionsFuncType)(EnvDoc *);
typedef short (CALLBACK* SetIdrisiExtensionsFuncType)(EnvDoc *);

// basic class member functions

Idrapp::Idrapp() {

   // Get a handle to the DLL module.

   hinstLib = LoadLibrary("mercur32");
   return;
}

Idrapp::~Idrapp() {

   if (hinstLib == NULL) return;
   BOOL fFreeResult = FreeLibrary(hinstLib);
   return ;
}

short Idrapp::IsIdrisiPresent() {

   IsIdrisiPresentFuncType f;

   if (hinstLib == NULL) return 0;
   f = (IsIdrisiPresentFuncType) GetProcAddress(hinstLib, "IsIdrisiPresent");
   if (!f) return 0;
   return f();
}

short Idrapp::LaunchIdrisi(short icmdshow) {

   LaunchIdrisiFuncType f;

   if (hinstLib == NULL) return 0;
   f = (LaunchIdrisiFuncType) GetProcAddress(hinstLib, "LaunchIdrisi");
   if (!f) return 0;
   return f(icmdshow);
}

short Idrapp::SetDataDirectory(char * DataDir) {

   SetDataDirectoryFuncType f;

   if (hinstLib == NULL) return 0;
   f = (SetDataDirectoryFuncType) GetProcAddress(hinstLib, "SetDataDirectory");
   if (!f) return 0;
   return f(DataDir);
}

short Idrapp::RegisterClient() {

   RegisterClientFuncType f;

   if (hinstLib == NULL) return 0;
   f = (RegisterClientFuncType) GetProcAddress(hinstLib, "RegisterClient");
   if (!f) return 0;
   return f();
}

short Idrapp::LaunchModule(short ClientId, short ClientOptions, char * ModName,
   char * Cmdln, char * OutputTitle, char * OutputUnits, short * PtHinst) {

   LaunchModuleFuncType f;

   if (hinstLib == NULL) return 0;
   f = (LaunchModuleFuncType) GetProcAddress(hinstLib, "LaunchModule");
   if (!f) return 0;
   return f(ClientId, ClientOptions, ModName, Cmdln, OutputTitle,
      OutputUnits, PtHinst);
}

short Idrapp::GetProgress(short ClientId, short ProcID, ProgStruct * Prog) {

   GetProgressFuncType f;

   if (hinstLib == NULL) return 0;
   f = (GetProgressFuncType) GetProcAddress(hinstLib, "GetProgress");
   if (!f) return 0;
   return f(ClientId, ProcID, Prog);
}

short Idrapp::UnRegisterClient(short ClientId) {

   UnRegisterClientFuncType f;

   if (hinstLib == NULL) return 0;
   f = (UnRegisterClientFuncType) GetProcAddress(hinstLib, "UnRegisterClient");
   if (!f) return 0;
   return f(ClientId);
}

short Idrapp::CloseIdrisi() {

   CloseIdrisiFuncType f;
   if (hinstLib == NULL) return 0;
   f = (CloseIdrisiFuncType) GetProcAddress(hinstLib, "CloseIdrisi");
   if (!f) return 0;
   return f();
}

// extended class member functions

short Idrapp::SetDebugMode(short mode) {

   SetDebugModeFuncType f;
   if (hinstLib == NULL) return 0;
   f = (SetDebugModeFuncType) GetProcAddress(hinstLib, "SetDebugMode");
   if (!f) return 0;
   return f(mode);
}

short Idrapp::GetIdrisiHwnd() {

   GetIdrisiHwndFuncType f;
   if (hinstLib == NULL) return 0;
   f = (GetIdrisiHwndFuncType) GetProcAddress(hinstLib, "GetIdrisiHwnd");
   if (!f) return 0;
   return f();
}

short Idrapp::GetIdrisiDirectory(char * IdrDir) {

   GetIdrisiDirectoryFuncType f;
   if (hinstLib == NULL) return 0;
   f = (GetIdrisiDirectoryFuncType) GetProcAddress(hinstLib, "GetIdrisiDirectory");
   if (!f) return 0;
   return f(IdrDir);
}

short Idrapp::GetIdrisiLanguage(char * IdrLang) {

   GetIdrisiLanguageFuncType f;
   if (hinstLib == NULL) return 0;
   f = (GetIdrisiLanguageFuncType) GetProcAddress(hinstLib, "GetIdrisiLanguage");
   if (!f) return 0;
   return f(IdrLang);
}

short Idrapp::GetDataDirectory(char * DataDir) {

   GetDataDirectoryFuncType f;

   if (hinstLib == NULL) return 0;
   f = (GetDataDirectoryFuncType) GetProcAddress(hinstLib, "GetDataDirectory");
   if (!f) return 0;
   return f(DataDir);
}

short Idrapp::InitProgressTracking(short ClientId, char * ProcName) {

   InitProgressTrackingFuncType f;

   if (hinstLib == NULL) return 0;
   f = (InitProgressTrackingFuncType) GetProcAddress(hinstLib, "InitProgressTracking");
   if (!f) return 0;
   return f(ClientId, ProcName);
}

short Idrapp::SetProgress(short ClientId, short ProcID, ProgStruct * Prog) {

   SetProgressFuncType f;

   if (hinstLib == NULL) return 0;
   f = (SetProgressFuncType) GetProcAddress(hinstLib, "SetProgress");
   if (!f) return 0;
   return f(ClientId, ProcID, Prog);
}

short Idrapp::DisplayComposition(short ClientId, short DispType,
   char * FName, char * PalName) {

   DisplayCompositionFuncType f;

   if (hinstLib == NULL) return 0;
   f = (DisplayCompositionFuncType) GetProcAddress(hinstLib, "DisplayComposition");
   if (!f) return 0;
   return f(ClientId, DispType, FName, PalName);
}

short Idrapp::Read_DocFile(char * FName, ImageDoc * Doc) {

   Read_DocFileFuncType f;

   if (hinstLib == NULL) return 0;
   f = (Read_DocFileFuncType) GetProcAddress(hinstLib, "Read_DocFile");
   if (!f) return 0;
   return f(FName, Doc);
}

short Idrapp::Create_DocFile(char * FName, ImageDoc * Doc) {

   Create_DocFileFuncType f;

   if (hinstLib == NULL) return 0;
   f = (Create_DocFileFuncType) GetProcAddress(hinstLib, "Create_DocFile");
   if (!f) return 0;
   return f(FName, Doc);
}

short Idrapp::Read_Vec_DocFile(char * FName, VectorDoc * Doc) {

   Read_Vec_DocFileFuncType f;

   if (hinstLib == NULL) return 0;
   f = (Read_Vec_DocFileFuncType) GetProcAddress(hinstLib, "Read_Vec_DocFile");
   if (!f) return 0;
   return f(FName, Doc);
}

short Idrapp::Create_Vec_DocFile(char * FName, VectorDoc * Doc) {

   Create_Vec_DocFileFuncType f;

   if (hinstLib == NULL) return 0;
   f = (Create_Vec_DocFileFuncType) GetProcAddress(hinstLib, "Create_Vec_DocFile");
   if (!f) return 0;
   return f(FName, Doc);
}

short Idrapp::Read_Legend(char * FName, short FileType, short LegendCat,
   char * LegendText) {

   Read_LegendFuncType f;

   if (hinstLib == NULL) return 0;
   f = (Read_LegendFuncType) GetProcAddress(hinstLib, "Read_Legend");
   if (!f) return 0;
   return f(FName, FileType, LegendCat, LegendText);
}

short Idrapp::Add_Legend(char * FName, short FileType, short LegendCat,
   char * LegendText) {

   Add_LegendFuncType f;

   if (hinstLib == NULL) return 0;
   f = (Add_LegendFuncType) GetProcAddress(hinstLib, "Add_Legend");
   if (!f) return 0;
   return f(FName, FileType, LegendCat, LegendText);
}

short Idrapp::GetIdrisiExtensions(EnvDoc * Ext) {

   GetIdrisiExtensionsFuncType f;

   if (hinstLib == NULL) return 0;
   f = (GetIdrisiExtensionsFuncType) GetProcAddress(hinstLib, "GetIdrisiExtensions");
   if (!f) return 0;
   return f(Ext);
}

short Idrapp::SetIdrisiExtensions(EnvDoc * Ext) {

   SetIdrisiExtensionsFuncType f;

   if (hinstLib == NULL) return 0;
   f = (SetIdrisiExtensionsFuncType) GetProcAddress(hinstLib, "SetIdrisiExtensions");
   if (!f) return 0;
   return f(Ext);
}

