// idrapp.h--declarations for Idrapp class

// basic constants

#define PROCSTATUS_ACTIVE 1
#define PROCSTATUS_NORMAL_TERMINATE 2
#define PROCSTATUS_ERROR_TERMINATE 3
#define REPORT_TYPE_WORKING 1
#define REPORT_TYPE_PERCENTDONE 2
#define REPORT_TYPE_PASS_X_OF_N 3
#define REPORT_TYPE_PASS_TOTALUNKNOWN 4
#define REPORT_TYPE_COMPLEXPASS 5
#define MONITOR_FROM_CLIENT_ONLY 0
#define MONITOR_FROM_IDRISI 1
#define SHOW_IDRISI_HIDDEN 0
#define SHOW_IDRISI_NORMAL 1
#define SHOW_IDRISI_MINIMIZED 2
#define SHOW_IDRISI_MAXIMIZED 3

// extended constants

#define DISPLAY_TYPE_IMAGE 0
#define DISPLAY_TYPE_VECTOR 1
#define DISPLAY_TYPE_COMP 2
#define DATA_TYPE_INTEGER 0
#define DATA_TYPE_REAL 1
#define DATA_TYPE_BYTE 2
#define FILE_TYPE_ASCII 0
#define FILE_TYPE_BINARY 1
#define FILE_TYPE_PACKED 2
#define OBJ_TYPE_POINT 0
#define OBJ_TYPE_LINE 1
#define OBJ_TYPE_POLYGON 2
#define OBJ_TYPE_ARC 3
#define OBJ_TYPE_COVERAGE 4
#define OBJ_TYPE_TEXT 6
#define OBJ_TYPE_UNKNOWN 999
#define DEBUG_MODE_OFF 0
#define DEBUG_MODE_ON 1
#define LEGEND_TYPE_IMAGE 0
#define LEGEND_TYPE_VALUE 1

// basic data structures

struct ProgStruct
{
  short Status;
  short ReportType;
  short PassNum;
  short TotalPasses;
  float PercentDone;
  short ErrorCode;
  char ErrorFile[256];
  char ErrorMessage[256];
  char SubstString1[256];
  char SubstString2[256];
};

// extended data structures

struct ImageDoc
{
  char Title[256];
  short DataType;
  short FileType;
  int Cols;
  int Rows;
  char RefSystem[256];
  char RefUnits[256];
  float UnitDistance;
  float MinX;
  float MaxX;
  float MinY;
  float MaxY;
  char PosnError[256];
  char Resolution[256];
  float MinValue;
  float MaxValue;
  char ValUnits[256];
  char ValError[256];
  char FlagValue[256];
  char FlagDefn[256];
  short LegendCats;
};
struct VectorDoc
{
  char title[256];
  short datatype;
  short filetype;
  short objtype;
  char refsystem[256];
  char refunits[256];
  float unitdist;
  float minX;
  float maxX;
  float minY;
  float maxY;
  char posnerror[256];
  char resolution[256];
};
struct FieldDoc
{
  char fieldname[256];
  short datatype;
  short format;
};
struct ValuesDoc
{
  char title[256];
  short filetype;
  short records;
  short numfields;
  FieldDoc field[2];
  float minvalue;
  float maxvalue;
  char valunits[256];
  char valerror[256];
  char flagvalue[256];
  char flagdefn[256];
  short LegendCats;
};
struct EnvDoc
{
  char img_file_extension[5];
  char imgdoc_file_extension[5];
  char vec_file_extension[5];
  char vecdoc_file_extension[5];
  char val_file_extension[5];
  char valdoc_file_extension[5];
};

// class declaration

class Idrapp
{
protected:
   HINSTANCE hinstLib;

public:

// basic member functions

   Idrapp();     // constructor (loads mercur32)
   short IsIdrisiPresent();
   short LaunchIdrisi(short);
   short SetDataDirectory(char *);
   short RegisterClient();
   short LaunchModule(short, short, char *, char *, char *, char *, short *);
   short GetProgress(short, short, ProgStruct *);
   short UnRegisterClient(short);
   short CloseIdrisi();
   ~Idrapp();    // destructor (unloads mercur32)

// extended member functions

   short SetDebugMode(short);
   short GetIdrisiHwnd();
   short GetIdrisiDirectory(char *);
   short GetIdrisiLanguage(char *);
   short GetDataDirectory(char *);
   short InitProgressTracking(short, char *);
   short SetProgress(short, short, ProgStruct *);
   short DisplayComposition(short, short, char *, char *);
   short Read_DocFile(char *, ImageDoc *);
   short Create_DocFile(char *, ImageDoc *);
   short Read_Vec_DocFile(char *, VectorDoc *);
   short Create_Vec_DocFile(char *, VectorDoc *);
   short Read_Val_DocFile(char *, ValuesDoc *);
   short Create_Val_DocFile(char *, ValuesDoc *);
   short Read_Legend(char *, short, short, char *);
   short Add_Legend(char *, short, short, char *);
   short GetIdrisiExtensions(EnvDoc *);
   short SetIdrisiExtensions(EnvDoc *);
};

